@@ -43,8 +43,8 @@ def build_sincos2d_pos_embed(
         temperature: float = 10000.,
         reverse_coord: bool = False,
         interleave_sin_cos: bool = False,
+        device: Optional[torch.device] = None,
         dtype: torch.dtype = torch.float32,
-        device: Optional[torch.device] = None
 ) -> torch.Tensor:
     """
 
@@ -96,8 +96,8 @@ def build_fourier_pos_embed(
         ref_feat_shape: Optional[List[int]] = None,
         grid_offset: float = 0.,
         grid_indexing: str = 'ij',
-        dtype: torch.dtype = torch.float32,
         device: Optional[torch.device] = None,
+        dtype: torch.dtype = torch.float32,
 ) -> List[torch.Tensor]:
     """
 
@@ -164,7 +164,7 @@ def build_fourier_pos_embed(
     grid = grid.unsqueeze(-1)
     pos = grid * bands
 
-    pos_sin, pos_cos = pos.sin().to(dtype=dtype), pos.cos().to(dtype)
+    pos_sin, pos_cos = pos.sin().to(dtype=dtype), pos.cos().to(dtype=dtype)
     out = [grid, pos_sin, pos_cos] if include_grid else [pos_sin, pos_cos]
     return out
 
@@ -177,6 +177,8 @@ class FourierEmbed(nn.Module):
             num_bands: int = 64,
             concat_grid=True,
             keep_spatial=False,
+            device=None,
+            dtype=None,
     ):
         super().__init__()
         self.max_res = max_res
@@ -185,7 +187,7 @@ class FourierEmbed(nn.Module):
         self.keep_spatial = keep_spatial
         self.register_buffer(
             'bands',
-            pixel_freq_bands(max_res, num_bands),
+            pixel_freq_bands(max_res, num_bands).to(device=device, dtype=dtype),
             persistent=False,
         )
 
@@ -214,24 +216,75 @@ class FourierEmbed(nn.Module):
 
 
 def rot(x):
+    # x:   [ x0  x1  x2  x3  x4  x5]
+    # out: [-x1  x0 -x3  x2 -x5  x4]
     return torch.stack([-x[..., 1::2], x[..., ::2]], -1).reshape(x.shape)
 
 
-def apply_rot_embed(x: torch.Tensor, sin_emb, cos_emb):
-    if sin_emb.ndim == 3:
-        return x * cos_emb.unsqueeze(1).expand_as(x) + rot(x) * sin_emb.unsqueeze(1).expand_as(x)
-    return x * cos_emb + rot(x) * sin_emb
+def rope_rotate_half(x: torch.Tensor) -> torch.Tensor:
+    # x:   [ x0  x1  x2  x3  x4  x5]
+    # out: [-x3 -x4 -x5  x0  x1  x2]
+    x1, x2 = x.chunk(2, dim=-1)
+    return torch.cat([-x2, x1], dim=-1)
 
 
-def apply_rot_embed_list(x: List[torch.Tensor], sin_emb, cos_emb):
+def apply_rot_embed(
+        x: torch.Tensor,
+        sin_emb: torch.Tensor,
+        cos_emb: torch.Tensor,
+        half: bool = False,
+) -> torch.Tensor:
+    # x: [..., D], eg [x0, x1, x2, x3, x4, x5]
+    if half:
+        # sin: [..., D], eg [sin0, sin1, sin2, sin0, sin1, sin2]
+        # cos: [..., D], eg [cos0, cos1, cos2, cos0, cos1, cos2
+        # rope_rotate_half(x): eg [-x3, -x4, -x5, x0, x1, x2]
+        return x * cos_emb + rope_rotate_half(x) * sin_emb
+    else:
+        # sin: [..., D], eg [sin0, sin0, sin1, sin1, sin2, sin2]
+        # cos: [..., D], eg [cos0, cos0, cos1, cos1, cos2, cos2]
+        # rot(x): eg [-x1, x0, -x3, x2, -x5, x4]
+        return x * cos_emb + rot(x) * sin_emb
+
+
+def apply_rot_embed_list(
+        x: List[torch.Tensor],
+        sin_emb: torch.Tensor,
+        cos_emb: torch.Tensor,
+        half: bool = False
+) -> List[torch.Tensor]:
     if isinstance(x, torch.Tensor):
         x = [x]
-    return [t * cos_emb + rot(t) * sin_emb for t in x]
+    # x: [..., D], eg [x0, x1, x2, x3, x4, x5]
+    if half:
+        # sin: [..., D], eg [sin0, sin1, sin2, sin0, sin1, sin2]
+        # cos: [..., D], eg [cos0, cos1, cos2, cos0, cos1, cos2
+        # rope_rotate_half(x): eg [-x3, -x4, -x5, x0, x1, x2]
+        return [t * cos_emb + rope_rotate_half(t) * sin_emb for t in x]
+    else:
+        # sin: [..., D], eg [sin0, sin0, sin1, sin1, sin2, sin2]
+        # cos: [..., D], eg [cos0, cos0, cos1, cos1, cos2, cos2]
+        # rot(x): eg [-x1, x0, -x3, x2, -x5, x4]
+        return [t * cos_emb + rot(t) * sin_emb for t in x]
 
 
-def apply_rot_embed_cat(x: torch.Tensor, emb):
-    sin_emb, cos_emb = emb.tensor_split(2, -1)
-    return x * cos_emb + rot(x) * sin_emb
+def apply_rot_embed_cat(
+        x: torch.Tensor,
+        emb: torch.Tensor,
+        half: bool = False
+) -> torch.Tensor:
+    sin_emb, cos_emb = emb.chunk(2, -1)
+    # x: [..., D], eg [x0, x1, x2, x3, x4, x5]
+    if half:
+        # sin: [..., D], eg [sin0, sin1, sin2, sin0, sin1, sin2]
+        # cos: [..., D], eg [cos0, cos1, cos2, cos0, cos1, cos2
+        # rope_rotate_half(x), eg [-x3, -x4, -x5, x0, x1, x2]
+        return x * cos_emb + rope_rotate_half(x) * sin_emb
+    else:
+        # sin: [..., D], eg [sin0, sin0, sin1, sin1, sin2, sin2]
+        # cos: [..., D], eg [cos0, cos0, cos1, cos1, cos2, cos2]
+        # rot(x), eg [-x1, x0, -x3, x2, -x5, x4]
+        return x * cos_emb + rot(x) * sin_emb
 
 
 def apply_keep_indices_nlc(
@@ -282,8 +335,8 @@ def build_rotary_pos_embed(
         ref_feat_shape: Optional[List[int]] = None,
         grid_offset: float = 0.,
         grid_indexing: str = 'ij',
-        dtype: torch.dtype = torch.float32,
         device: Optional[torch.device] = None,
+        dtype: torch.dtype = torch.float32,
 ):
     """
 
@@ -298,8 +351,8 @@ def build_rotary_pos_embed(
         ref_feat_shape: Reference feature shape for resize / fine-tune.
         grid_offset: Constant offset to add to grid for non-pixel freq.
         grid_indexing: Indexing mode for meshgrid ('ij' or 'xy')
-        dtype: Output dtype.
         device: Output device.
+        dtype: Output dtype.
 
     Returns:
 
@@ -349,6 +402,8 @@ class RotaryEmbedding(nn.Module):
             ref_feat_shape: Optional[List[int]] = None,
             grid_offset: float = 0.,
             grid_indexing: str = 'ij',
+            device=None,
+            dtype=None,
     ):
         super().__init__()
         self.dim = dim
@@ -377,14 +432,14 @@ class RotaryEmbedding(nn.Module):
                 )
             self.register_buffer(
                 'bands',
-                bands,
+                bands.to(device=device, dtype=dtype),
                 persistent=False,
             )
             self.pos_embed_sin = None
             self.pos_embed_cos = None
         else:
             # cache full sin/cos embeddings if shape provided up front
-            emb_sin, emb_cos = self._get_pos_embed_values(feat_shape)
+            emb_sin, emb_cos = self._get_pos_embed_values(feat_shape, device=device, dtype=dtype)
             self.bands = None
             self.register_buffer(
                 'pos_embed_sin',
@@ -397,7 +452,7 @@ class RotaryEmbedding(nn.Module):
                 persistent=False,
             )
 
-    def _get_pos_embed_values(self, feat_shape: List[int]):
+    def _get_pos_embed_values(self, feat_shape: List[int], device=None, dtype=torch.float32):
         emb_sin, emb_cos = build_rotary_pos_embed(
             feat_shape=feat_shape,
             dim=self.dim,
@@ -408,6 +463,8 @@ class RotaryEmbedding(nn.Module):
             ref_feat_shape=self.ref_feat_shape,
             grid_offset=self.grid_offset,
             grid_indexing=self.grid_indexing,
+            device=device,
+            dtype=dtype,
         )
         return emb_sin, emb_cos
 
@@ -416,9 +473,11 @@ class RotaryEmbedding(nn.Module):
             # only update if feat_shape was set and different from previous value
             assert self.pos_embed_sin is not None
             assert self.pos_embed_cos is not None
-            emb_sin, emb_cos = self._get_pos_embed_values(feat_shape)
-            self.pos_embed_sin = emb_sin.to(self.pos_embed_sin.device, self.pos_embed_sin.dtype)
-            self.pos_embed_cos = emb_cos.to(self.pos_embed_cos.device, self.pos_embed_cos.dtype)
+            self.pos_embed_sin, self.pos_embed_cos = self._get_pos_embed_values(
+                feat_shape,
+                device=self.pos_embed_sin.device,
+                dtype=self.pos_embed_sin.dtype,
+            )
             self.feat_shape = feat_shape
 
     def get_embed(self, shape: Optional[List[int]] = None):
@@ -453,15 +512,17 @@ class RotaryEmbeddingCat(nn.Module):
 
     def __init__(
             self,
-            dim,
-            max_res=224,
-            temperature=10000,
-            in_pixels=True,
+            dim: int,
+            max_res: int = 224,
+            temperature: float = 10000,
+            in_pixels: bool = True,
             linear_bands: bool = False,
             feat_shape: Optional[List[int]] = None,
             ref_feat_shape: Optional[List[int]] = None,
             grid_offset: float = 0.,
             grid_indexing: str = 'ij',
+            device=None,
+            dtype=None,
     ):
         super().__init__()
         self.dim = dim
@@ -490,7 +551,7 @@ class RotaryEmbeddingCat(nn.Module):
                 )
             self.register_buffer(
                 'bands',
-                bands,
+                bands.to(device=device, dtype=dtype),
                 persistent=False,
             )
             self.pos_embed = None
@@ -499,11 +560,11 @@ class RotaryEmbeddingCat(nn.Module):
             self.bands = None
             self.register_buffer(
                 'pos_embed',
-                self._get_pos_embed_values(feat_shape=feat_shape),
+                self._get_pos_embed_values(feat_shape=feat_shape, device=device, dtype=dtype),
                 persistent=False,
             )
 
-    def _get_pos_embed_values(self, feat_shape: List[int]):
+    def _get_pos_embed_values(self, feat_shape: List[int], device=None, dtype=torch.float32):
         embeds = build_rotary_pos_embed(
             feat_shape=feat_shape,
             dim=self.dim,
@@ -514,6 +575,8 @@ class RotaryEmbeddingCat(nn.Module):
             ref_feat_shape=self.ref_feat_shape,
             grid_offset=self.grid_offset,
             grid_indexing=self.grid_indexing,
+            device=device,
+            dtype=dtype,
         )
         return torch.cat(embeds, -1)
 
@@ -521,7 +584,8 @@ class RotaryEmbeddingCat(nn.Module):
         if self.feat_shape is not None and feat_shape != self.feat_shape:
             # only update if feat_shape was set and different from previous value
             assert self.pos_embed is not None
-            self.pos_embed = self._get_pos_embed_values(feat_shape).to(
+            self.pos_embed = self._get_pos_embed_values(
+                feat_shape,
                 device=self.pos_embed.device,
                 dtype=self.pos_embed.dtype,
             )
@@ -648,12 +712,12 @@ def get_mixed_grid(
     if grid_indexing == 'xy':
         shape = swap_shape_xy(shape)
     x_pos, y_pos = torch.meshgrid(
-        torch.arange(shape[0], dtype=dtype, device=device),
-        torch.arange(shape[1], dtype=dtype, device=device),
+        torch.arange(shape[0], device=device, dtype=torch.float32),
+        torch.arange(shape[1], device=device, dtype=torch.float32),
         indexing=grid_indexing,
     )
-    t_x = x_pos.flatten()
-    t_y = y_pos.flatten()
+    t_x = x_pos.to(dtype).flatten()
+    t_y = y_pos.to(dtype).flatten()
     return t_x, t_y
 
 
@@ -692,6 +756,8 @@ class RotaryEmbeddingMixed(nn.Module):
             temperature: float = 10.0,
             feat_shape: Optional[List[int]] = None,
             grid_indexing: str = 'xy',
+            device=None,
+            dtype=None,
     ):
         """Initialize rotary embeddings.
 
@@ -720,6 +786,8 @@ class RotaryEmbeddingMixed(nn.Module):
             num_heads,
             temperature=temperature,
             rotate=True,
+            device=device,
+            dtype=dtype,
         )  # (2, depth, num_heads, head_dim//2)
         self.freqs = nn.Parameter(freqs)
 
@@ -735,7 +803,7 @@ class RotaryEmbeddingMixed(nn.Module):
         t_x, t_y = get_mixed_grid(
             feat_shape,
             grid_indexing=self.grid_indexing,
-            device=self.freqs.device
+            device=self.freqs.device,
         )
         return t_x, t_y
 
@@ -834,3 +902,275 @@ class RotaryEmbeddingMixed(nn.Module):
     def no_weight_decay(self):
         """Exclude frequency parameters from weight decay."""
         return {'freqs'}
+
+
+@torch.fx.wrap
+@register_notrace_function
+def make_coords_dinov3(
+        height: int,
+        width: int,
+        normalize_coords: str = 'separate',
+        grid_indexing: str = 'ij',
+        grid_offset: float = 0.,
+        device: torch.device = 'cpu',
+        dtype: torch.dtype = torch.float32,
+) -> torch.Tensor:
+    """Make coordinate grid matching offset and normalization of original.
+    Returns: coords with shape (HW, 2) in [-1, 1].
+    """
+    # 0.5-centered indices with optional offset
+    coords_h = torch.arange(0.5, height, device=device, dtype=torch.float32) + grid_offset
+    coords_w = torch.arange(0.5, width, device=device, dtype=torch.float32) + grid_offset
+
+    # Normalization denominators
+    if normalize_coords == "max":
+        denom = float(max(height, width))
+        h_denom = denom
+        w_denom = denom
+    elif normalize_coords == "min":
+        denom = float(min(height, width))
+        h_denom = denom
+        w_denom = denom
+    elif normalize_coords == "separate":
+        h_denom = float(height)
+        w_denom = float(width)
+    else:
+        raise ValueError(f"Unknown normalize_coords: {normalize_coords}")
+
+    # Normalize to [0, 1]
+    coords_h = coords_h / h_denom
+    coords_w = coords_w / w_denom
+    coords_h = coords_h.to(dtype)
+    coords_w = coords_w.to(dtype)
+
+    # Create grid then map to [-1, 1]
+    if grid_indexing == "xy":
+        grid_w, grid_h = torch.meshgrid(coords_w, coords_h, indexing="xy")
+        coords = torch.stack([grid_h, grid_w], dim=-1)  # (H, W, 2) -> (h, w order)
+    else:
+        coords = torch.stack(torch.meshgrid(coords_h, coords_w, indexing="ij"), dim=-1)  # (H, W, 2)
+    coords = coords.flatten(0, 1)  # (HW, 2)
+    coords = 2.0 * coords - 1.0  # (H, W, 2) in [-1, 1]
+    return coords
+
+
+class RotaryEmbeddingDinoV3(nn.Module):
+    """RoPE for timm DinoV3 port, numerically matching original.
+
+    Math is aligned to original DinoV3 RopePositionEmbedding at https://github.com/facebookresearch/dinov3:
+      - 0.5-centered coords normalized by H/W (or min/max), mapped to [-1,1]
+      - training-time augmentations (shift/jitter/rescale)
+      - periods schedule equals Rope's temperature (base) or min/max period
+    """
+
+    def __init__(
+            self,
+            dim: int,
+            temperature: Optional[float] = 100.0,
+            min_period: Optional[float] = None,
+            max_period: Optional[float] = None,
+            feat_shape: Optional[List[int]] = None,
+            normalize_coords: str = "separate",  # 'min', 'max', 'separate'
+            grid_offset: float = 0.0,
+            grid_indexing: str = "ij",
+            rotate_half: bool = True,
+            shift_coords: Optional[float] = None,
+            jitter_coords: Optional[float] = None,  # interpreted as factor J >= 1
+            rescale_coords: Optional[float] = None,  # interpreted as factor R >= 1
+            device=None,
+            dtype=None,
+    ):
+        super().__init__()
+
+        # Dimensions / output format
+        self.dim = dim  # equal to head_dim for most vit applications
+        self.rotate_half = rotate_half
+
+        # Period schedule parameters
+        self.temperature = float(temperature)
+        self.min_period = min_period
+        self.max_period = max_period
+
+        # Coord processing + augs
+        self.normalize_coords = normalize_coords
+        self.shift_coords = shift_coords
+        self.jitter_coords = jitter_coords
+        self.rescale_coords = rescale_coords
+        self.aug_active = any([a is not None for a in [self.shift_coords, self.jitter_coords, self.rescale_coords]])
+
+        # Grid config
+        self.feat_shape = feat_shape
+        self.grid_offset = grid_offset
+        self.grid_indexing = grid_indexing
+
+        # Precompute periods
+        periods = self._compute_periods(device=device, dtype=dtype)
+        self.register_buffer("periods", periods, persistent=False)
+
+        if feat_shape is not None:
+            self._cache_embed(feat_shape)
+        else:
+            self.register_buffer("pos_embed_cached", None, persistent=False)
+            self.feat_shape = None
+
+    def _compute_periods(self, device: torch.device = 'cpu', dtype: torch.dtype = torch.float32) -> torch.Tensor:
+        """Construct periods from either min/max or temperature."""
+        dim = self.dim // 4
+
+        if self.min_period is not None and self.max_period is not None:
+            exponents = torch.linspace(0, 1, dim, device='cpu', dtype=torch.float32)
+            periods = self.min_period * ((self.max_period / self.min_period) ** exponents)
+        else:
+            if self.temperature is None:
+                raise ValueError("Provide either min/max periods or `temperature`.")
+            exponents = 2.0 * torch.arange(dim, device='cpu', dtype=torch.float32) / (self.dim // 2)
+            periods = self.temperature ** exponents
+
+        # NOTE: The original dinv3 model weights have periods downcast to bfloat16 in persistent buffers,
+        # loaded models will differ a bit vs timm as periods is not persistent and generated in float32 by default
+        return periods.to(device=device, dtype=dtype)
+
+    def _apply_coord_augs(self, coords: torch.Tensor) -> torch.Tensor:
+        """Apply shift/jitter/rescale train time augmentations."""
+        if not self.training or not self.aug_active:
+            return coords
+
+        device = coords.device
+        dtype = coords.dtype
+
+        # Shift per-axis in [-s, +s]
+        if self.shift_coords is not None:
+            shift = float(self.shift_coords)
+            shift_hw = torch.empty(2, device=device, dtype=dtype).uniform_(-shift, shift)
+            coords = coords + shift_hw[None, :]
+
+        # Jitter: per-axis log-uniform factor in [1/J, J]
+        if self.jitter_coords is not None:
+            jitter_factor = float(self.jitter_coords)
+            if jitter_factor <= 0:
+                raise ValueError("jitter_coords must be > 0 (interpreted as multiplicative factor).")
+            jitter_max = math.log(jitter_factor)
+            jitter_hw = torch.empty(2, device=device, dtype=dtype).uniform_(-jitter_max, jitter_max).exp()
+            coords = coords * jitter_hw[None, :]
+
+        # Rescale: shared scalar log-uniform factor in [1/R, R]
+        if self.rescale_coords is not None:
+            rescale_factor = float(self.rescale_coords)
+            if rescale_factor <= 0:
+                raise ValueError("rescale_coords must be > 0 (interpreted as multiplicative factor).")
+            rescale_max = math.log(rescale_factor)
+            rescale = torch.empty(1, device=device, dtype=dtype).uniform_(-rescale_max, rescale_max).exp()
+            coords = coords * rescale
+
+        return coords
+
+    def _get_pos_embed_from_coords(self, coords: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
+        """Return sin/cos embeddings with either 'half' or 'interleaved' layout."""
+        # coords: (HW, 2); periods: (dim)
+        dim = self.dim // 4
+        device = self.periods.device
+        dtype = self.periods.dtype
+        assert self.periods.numel() == dim
+
+        # NOTE this is a slightly later device/dtype switch than original
+        coords = coords[:, :, None].to(device=device, dtype=dtype)
+        angles = 2 * math.pi * coords / self.periods[None, None, :]
+        angles = angles.flatten(1)  # (HW, dim // 2)
+
+        if self.rotate_half:
+            # Tile (half layout) (HW, dim // 2) -> (HW, dim)
+            angles = angles.tile(2)
+        else:
+            # Interleaved layout (HW, dim // 2) -> (HW, dim)
+            angles = angles.repeat_interleave(2, dim=-1)
+
+        sin = torch.sin(angles)
+        cos = torch.cos(angles)
+        return sin, cos
+
+    def _create_embed(
+            self,
+            feat_shape: List[int],
+            no_aug: bool = False,
+    ) -> torch.Tensor:
+        H, W = feat_shape
+        coords = make_coords_dinov3(
+            H, W,
+            normalize_coords=self.normalize_coords,
+            grid_indexing=self.grid_indexing,
+            grid_offset=self.grid_offset,
+        )  # (HW, 2)
+        if not no_aug:
+            coords = self._apply_coord_augs(coords)
+        sin, cos = self._get_pos_embed_from_coords(coords)  # 2 * (HW, dim)
+        rope_embed = torch.cat([sin, cos], dim=-1)  # (HW, 2*dim)
+        return rope_embed
+
+    def _cache_embed(self, feat_shape: List[int]):
+        # create non-augmented embeds for cache
+        rope_embed = self._create_embed(feat_shape, no_aug=True)
+        self.register_buffer("pos_embed_cached", rope_embed, persistent=False)
+        self.feat_shape = feat_shape
+
+    def update_feat_shape(self, feat_shape: List[int]):
+        if self.feat_shape is not None and feat_shape != self.feat_shape:
+            # only update if feat_shape was set (valid cache) and different from previous value
+            self._cache_embed(feat_shape)
+
+    def get_embed(self, shape: Optional[List[int]] = None) -> torch.Tensor:
+        """Generate rope_embed matching DINOv3 RopePositionEmbedding numerics.
+
+        Returns: (HW, num_heads, 2 * head_dim) with last dim = [sin, cos] cat.
+        """
+        if shape is not None:
+            rope_embed = self._create_embed(shape)
+        else:
+            need_create = self.pos_embed_cached is None or (self.training and self.aug_active)
+            if need_create:
+                assert self.feat_shape is not None, 'feature shape must be cached on create'
+                rope_embed = self._create_embed(self.feat_shape)
+            else:
+                assert self.pos_embed_cached is not None
+                rope_embed = self.pos_embed_cached
+
+        return rope_embed
+
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
+        """Get and apply rotary embeddings to x"""
+        # assuming channel-first tensor where spatial dim are >= 2
+        pos_embed = self.get_embed(x.shape[2:])
+        return apply_rot_embed_cat(x, pos_embed, half=self.rotate_half)
+
+
+def create_rope_embed(
+        rope_type: str = 'cat',
+        dim: int = 768,
+        num_heads: int = 12,
+        **kwargs
+) -> nn.Module:
+    """Factory function for creating rotary position embeddings.
+
+    Args:
+        rope_type: Type of RoPE to create. Options:
+            - 'base': Basic RotaryEmbedding
+            - 'cat': RotaryEmbeddingCat (concatenated sin/cos)
+            - 'mixed': RotaryEmbeddingMixed (learnable per-depth frequencies)
+            - 'dinov3': RotaryEmbeddingDinoV3 (with coordinate transforms)
+        dim: Total embedding dimension
+        num_heads: Number of attention heads
+        **kwargs: Additional arguments passed to the specific RoPE class
+
+    Returns:
+        Rotary embedding module
+    """
+    if rope_type == 'base':
+        return RotaryEmbedding(dim=dim // num_heads, **kwargs)
+    elif rope_type == 'cat':
+        return RotaryEmbeddingCat(dim=dim // num_heads, **kwargs)
+    elif rope_type == 'mixed':
+        # Mixed requires depth parameter, generates differing embeddings per layer and head
+        return RotaryEmbeddingMixed(dim=dim, num_heads=num_heads, **kwargs)
+    elif rope_type == 'dinov3':
+        return RotaryEmbeddingDinoV3(dim=dim // num_heads, **kwargs)
+    else:
+        raise ValueError(f"Unknown RoPE type: {rope_type}")
